/*==============================================================*/
/* Suppression des anciennes tables si elles existent           */
/*==============================================================*/

DROP TABLE IF EXISTS FIL CASCADE;
DROP TABLE IF EXISTS MESSAGE CASCADE;
DROP TABLE IF EXISTS PARTICIPANT_FIL CASCADE;
DROP TABLE IF EXISTS LIKES CASCADE;
DROP TABLE IF EXISTS UTILISATEUR CASCADE;

/*==============================================================*/
/* Création de la table UTILISATEUR                             */
/*==============================================================*/

CREATE TABLE UTILISATEUR (
   U_ID                 SERIAL               NOT NULL,
   PRENOM               TEXT                 NOT NULL,
   NOM                  TEXT                 NOT NULL,
   MAIL                 TEXT                 NOT NULL UNIQUE,
   MDP                  TEXT                 NOT NULL,
   CONSTRAINT PK_UTILISATEUR PRIMARY KEY (U_ID)
);

\! echo 'Création de la table UTILISATEUR avec succès'

/*==============================================================*/
/* Création de la table FIL                                     */
/*==============================================================*/

CREATE TABLE FIL (
   F_ID                 SERIAL               NOT NULL,
   TITRE                TEXT                 NULL,
   CONSTRAINT PK_FIL PRIMARY KEY (F_ID)
);

\! echo 'Création de la table FIL avec succès'

/*==============================================================*/
/* Création de la table MESSAGE                                 */
/*==============================================================*/

CREATE TABLE MESSAGE (
   M_ID                 SERIAL               NOT NULL,
   F_ID                 INT4                 NOT NULL,
   U_ID                 INT4                 NOT NULL,
   MESSAGE_DATA         BYTEA                 NULL,
   is_image             BOOLEAN              NULL,
   D_VIE                INT4                 NULL,
   DateMessage          TIMESTAMP            NULL,
   CONSTRAINT PK_MESSAGE PRIMARY KEY (M_ID),
   CONSTRAINT FK_MESSAGE_ENVOIE_ME_UTILISAT FOREIGN KEY (U_ID)
      REFERENCES UTILISATEUR (U_ID)
      ON DELETE CASCADE ON UPDATE CASCADE,
   CONSTRAINT FK_MESSAGE_MESSAGE_F_FIL FOREIGN KEY (F_ID)
      REFERENCES FIL (F_ID)
      ON DELETE CASCADE ON UPDATE CASCADE
);

\! echo 'Création de la table MESSAGE avec succès'

/*==============================================================*/
/* Création de la table PARTICIPANT_FIL                         */
/*==============================================================*/

CREATE TABLE PARTICIPANT_FIL (
   UP_ID                INT4                 NOT NULL,
   FP_ID                INT4                 NOT NULL,
   ADMINISTRATEUR       BOOL                 NULL,
   CONSTRAINT PK_PARTICIPANT_FIL PRIMARY KEY (UP_ID, FP_ID),
   CONSTRAINT FK_PARTICIP_PARTICIPA_UTILISAT FOREIGN KEY (UP_ID)
      REFERENCES UTILISATEUR (U_ID)
      ON DELETE CASCADE ON UPDATE CASCADE,
   CONSTRAINT FK_PARTICIP_PARTICIPA_FIL FOREIGN KEY (FP_ID)
      REFERENCES FIL (F_ID)
      ON DELETE CASCADE ON UPDATE CASCADE
);

\! echo 'Création de la table PARTICIPANT_FIL avec succès'

/*==============================================================*/
/* Création de la table LIKES                            */
/*==============================================================*/

CREATE TABLE LIKES (
   M_ID                 INT4                 NOT NULL,
   U_ID                 INT4                 NOT NULL,
   CONSTRAINT PK_LIKES PRIMARY KEY (M_ID, U_ID),
   CONSTRAINT FK_LIKES_MESSAGE FOREIGN KEY (M_ID)
      REFERENCES MESSAGE (M_ID)
      ON DELETE CASCADE ON UPDATE CASCADE,
   CONSTRAINT FK_LIKES_UTILISAT FOREIGN KEY (U_ID)
      REFERENCES UTILISATEUR (U_ID)
      ON DELETE CASCADE ON UPDATE CASCADE
);

\! echo 'Création de la table LIKES avec succès'

/*==============================================================*/
/* Insérer des utilisateurs                                     */
/*==============================================================*/

INSERT INTO UTILISATEUR (PRENOM, NOM, MAIL, MDP)
VALUES ('Tom', 'Dequesnes', 'tom.dequesnes@univ-lille.fr', MD5('totow'));
INSERT INTO UTILISATEUR (PRENOM, NOM, MAIL, MDP)
VALUES ('Giorgio', 'Utzeri', 'giorgio.utzeri@univ-lille.fr', MD5('giou'));

\! echo 'Insertion des premiers utilisateurs avec succès'

/*==============================================================*/
/* Insérer des fils pour chaque utilisateur                     */
/*==============================================================*/

/* Fil où chaque utilisateur est seul */
-- Fil 1 : Tom seul
INSERT INTO FIL (TITRE) VALUES ('Fil de Tom seul');
-- Fil 2 : Giorgio seul
INSERT INTO FIL (TITRE) VALUES ('Fil de Giorgio seul');

/* Fil où les utilisateurs sont tous les deux */
-- Fil 3 : Tom et Giorgio ensemble
INSERT INTO FIL (TITRE) VALUES ('Fil de discussion entre Tom et Giorgio');

/* Ajout des participants aux fils */
-- Tom participe au fil de Tom seul
INSERT INTO PARTICIPANT_FIL (UP_ID, FP_ID, ADMINISTRATEUR)
VALUES ((SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Tom' AND NOM = 'Dequesnes'), (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de Tom seul'), TRUE);

-- Giorgio participe au fil de Giorgio seul
INSERT INTO PARTICIPANT_FIL (UP_ID, FP_ID, ADMINISTRATEUR)
VALUES ((SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Giorgio' AND NOM = 'Utzeri'), (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de Giorgio seul'), TRUE);

-- Tom et Giorgio participent au fil ensemble
INSERT INTO PARTICIPANT_FIL (UP_ID, FP_ID, ADMINISTRATEUR)
VALUES ((SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Tom' AND NOM = 'Dequesnes'), (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de discussion entre Tom et Giorgio'), TRUE);

INSERT INTO PARTICIPANT_FIL (UP_ID, FP_ID, ADMINISTRATEUR)
VALUES ((SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Giorgio' AND NOM = 'Utzeri'), (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de discussion entre Tom et Giorgio'), TRUE);

\! echo 'Insertion de fils de test avec succès'

/*==============================================================*/
/* Insérer des messages pour chaque utilisateur dans les fils   */
/*==============================================================*/

/* Tom dans son fil */
INSERT INTO MESSAGE (F_ID, U_ID, MESSAGE_DATA, IS_IMAGE, d_vie, DateMessage)
VALUES (
    (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de Tom seul'),
    (SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Tom' AND NOM = 'Dequesnes'),
    convert_to('Message 1 de Tom', 'UTF8'), FALSE, 14, '2025-03-17 00:00:01'
);

INSERT INTO MESSAGE (F_ID, U_ID, MESSAGE_DATA, IS_IMAGE, d_vie, DateMessage)
VALUES (
    (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de Tom seul'),
    (SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Tom' AND NOM = 'Dequesnes'),
    convert_to('Message 2 de Tom', 'UTF8'), FALSE, 14, '2025-03-17 00:00:02'
);

/* Giorgio dans son fil */
INSERT INTO MESSAGE (F_ID, U_ID, MESSAGE_DATA, IS_IMAGE, d_vie, DateMessage)
VALUES (
    (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de Giorgio seul'),
    (SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Giorgio' AND NOM = 'Utzeri'),
    convert_to('Message 1 de Giorgio', 'UTF8'), FALSE, 14, '2025-03-17 00:00:03'
);

INSERT INTO MESSAGE (F_ID, U_ID, MESSAGE_DATA, IS_IMAGE, d_vie, DateMessage)
VALUES (
    (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de Giorgio seul'),
    (SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Giorgio' AND NOM = 'Utzeri'),
    convert_to('Message 2 de Giorgio', 'UTF8'), FALSE, 14, '2025-03-17 00:00:04'
);

/* Tom et Giorgio dans le fil commun */
INSERT INTO MESSAGE (F_ID, U_ID, MESSAGE_DATA, IS_IMAGE, d_vie, DateMessage)
VALUES (
    (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de discussion entre Tom et Giorgio'),
    (SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Tom' AND NOM = 'Dequesnes'),
    convert_to('Message 1 de Tom dans le fil commun', 'UTF8'), FALSE, 14, '2025-03-17 00:00:05'
);

INSERT INTO MESSAGE (F_ID, U_ID, MESSAGE_DATA, IS_IMAGE, d_vie, DateMessage)
VALUES (
    (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de discussion entre Tom et Giorgio'),
    (SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Giorgio' AND NOM = 'Utzeri'),
    convert_to('Message 1 de Giorgio dans le fil commun', 'UTF8'), FALSE, 14, '2025-03-17 00:00:06'
);

\! echo 'Création de message de test avec succès'

/*==============================================================*/
/* Ajouter un like d'un utilisateur sur un message de l'autre   */
/*==============================================================*/

/* Giorgio aime un message de Tom dans le fil commun */
INSERT INTO LIKES (M_ID, U_ID)
VALUES ((SELECT M_ID FROM MESSAGE WHERE F_ID = (SELECT F_ID FROM FIL WHERE TITRE = 'Fil de discussion entre Tom et Giorgio') AND U_ID = (SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Tom' AND NOM = 'Dequesnes') LIMIT 1), (SELECT U_ID FROM UTILISATEUR WHERE PRENOM = 'Giorgio' AND NOM = 'Utzeri'));

\! echo "Ajout d'un like sur un message avec succès"

/*==============================================================*/
/* Terminé                                                      */
/*==============================================================*/
